@@ -94,7 +94,7 @@ static int tdb_new_database(struct tdb_context *tdb, struct tdb_header *header,
 	}
 
 	/*
-	 * It's required for some following code pathes
+	 * It's required for some following code paths
 	 * to have the fields on 'tdb' up-to-date.
 	 *
 	 * E.g. tdb_mutex_size() requires it
@@ -230,8 +230,6 @@ static bool check_header_hash(struct tdb_context *tdb,
 static bool tdb_mutex_open_ok(struct tdb_context *tdb,
 			      const struct tdb_header *header)
 {
-	int locked;
-
 	if (tdb->flags & TDB_NOLOCK) {
 		/*
 		 * We don't look at locks, so it does not matter to have a
@@ -240,37 +238,6 @@ static bool tdb_mutex_open_ok(struct tdb_context *tdb,
 		return true;
 	}
 
-	locked = tdb_nest_lock(tdb, ACTIVE_LOCK, F_WRLCK,
-			       TDB_LOCK_NOWAIT|TDB_LOCK_PROBE);
-
-	if ((locked == -1) && (tdb->ecode == TDB_ERR_LOCK)) {
-		/*
-		 * CLEAR_IF_FIRST still active. The tdb was created on this
-		 * host, so we can assume the mutex implementation is
-		 * compatible. Important for tools like tdbdump on a still
-		 * open locking.tdb.
-		 */
-		goto check_local_settings;
-	}
-
-	/*
-	 * We got the CLEAR_IF_FIRST lock. That means the database was
-	 * potentially copied from somewhere else. The mutex implementation
-	 * might be incompatible.
-	 */
-
-	if (tdb_nest_unlock(tdb, ACTIVE_LOCK, F_WRLCK, false) == -1) {
-		/*
-		 * Should not happen
-		 */
-		TDB_LOG((tdb, TDB_DEBUG_ERROR, "tdb_mutex_open_ok: "
-			 "failed to release ACTIVE_LOCK on %s: %s\n",
-			 tdb->name, strerror(errno)));
-		return false;
-	}
-
-check_local_settings:
-
 	if (!(tdb->flags & TDB_MUTEX_LOCKING)) {
 		TDB_LOG((tdb, TDB_DEBUG_ERROR, "tdb_mutex_open_ok[%s]: "
 			 "Can use mutexes only with "
@@ -281,10 +248,10 @@ check_local_settings:
 
 	if (tdb_mutex_size(tdb) != header->mutex_size) {
 		TDB_LOG((tdb, TDB_DEBUG_ERROR, "tdb_mutex_open_ok[%s]: "
-			 "Mutex size changed from %u to %u\n.",
+			 "Mutex size changed from %"PRIu32" to %zu\n.",
 			 tdb->name,
-			 (unsigned int)header->mutex_size,
-			 (unsigned int)tdb_mutex_size(tdb)));
+			 header->mutex_size,
+			 tdb_mutex_size(tdb)));
 		return false;
 	}
 
@@ -297,7 +264,9 @@ _PUBLIC_ struct tdb_context *tdb_open_ex(const char *name, int hash_size, int td
 				tdb_hash_func hash_fn)
 {
 	int orig_errno = errno;
-	struct tdb_header header;
+	struct tdb_header header = {
+		.version = 0,
+	};
 	struct tdb_context *tdb;
 	struct stat st;
 	int rev = 0;
@@ -309,8 +278,6 @@ _PUBLIC_ struct tdb_context *tdb_open_ex(const char *name, int hash_size, int td
 	uint32_t magic1, magic2;
 	int ret;
 
-	ZERO_STRUCT(header);
-
 	if (!(tdb = (struct tdb_context *)calloc(1, sizeof *tdb))) {
 		/* Can't log this */
 		errno = ENOMEM;
@@ -418,14 +385,6 @@ _PUBLIC_ struct tdb_context *tdb_open_ex(const char *name, int hash_size, int td
 		 * the runtime check for existing tdb's comes later.
 		 */
 
-		if (!(tdb->flags & TDB_CLEAR_IF_FIRST)) {
-			TDB_LOG((tdb, TDB_DEBUG_ERROR, "tdb_open_ex: "
-				"invalid flags for %s - TDB_MUTEX_LOCKING "
-				"requires TDB_CLEAR_IF_FIRST\n", name));
-			errno = EINVAL;
-			goto fail;
-		}
-
 		if (tdb->flags & TDB_INTERNAL) {
 			TDB_LOG((tdb, TDB_DEBUG_ERROR, "tdb_open_ex: "
 				"invalid flags for %s - TDB_MUTEX_LOCKING and "
@@ -554,7 +513,13 @@ _PUBLIC_ struct tdb_context *tdb_open_ex(const char *name, int hash_size, int td
 
 	errno = 0;
 	if (read(tdb->fd, &header, sizeof(header)) != sizeof(header)
-	    || strcmp(header.magic_food, TDB_MAGIC_FOOD) != 0) {
+	    /*
+	     * Call strncmp() rather than strcmp() in case header.magic_food is
+	     * not zero‐terminated. We’re still checking the full string for
+	     * equality, as tdb_header::magic_food is larger than
+	     * TDB_MAGIC_FOOD.
+	     */
+	    || strncmp(header.magic_food, TDB_MAGIC_FOOD, sizeof(header.magic_food)) != 0) {
 		if (!(open_flags & O_CREAT) ||
 		    tdb_new_database(tdb, &header, hash_size) == -1) {
 			if (errno == 0) {
@@ -634,6 +599,30 @@ _PUBLIC_ struct tdb_context *tdb_open_ex(const char *name, int hash_size, int td
 		 * mutex locking.
 		 */
 		tdb->hdr_ofs = header.mutex_size;
+
+		if ((!(tdb_flags & TDB_CLEAR_IF_FIRST)) && (!tdb->read_only)) {
+			/*
+			 * Open an existing mutexed tdb, but without
+			 * CLEAR_IF_FIRST. We need to initialize the
+			 * mutex array and keep the CLEAR_IF_FIRST
+			 * lock locked.
+			 */
+			ret = tdb_nest_lock(tdb, ACTIVE_LOCK, F_WRLCK,
+					    TDB_LOCK_NOWAIT|TDB_LOCK_PROBE);
+			locked = (ret == 0);
+
+			if (locked) {
+				ret = tdb_mutex_init(tdb);
+				if (ret == -1) {
+					TDB_LOG((tdb,
+						 TDB_DEBUG_FATAL,
+						 "tdb_open_ex: tdb_mutex_init "
+						 "failed for ""%s: %s\n",
+						 name, strerror(errno)));
+					goto fail;
+				}
+			}
+		}
 	}
 
 	if ((header.magic1_hash == 0) && (header.magic2_hash == 0)) {
@@ -672,7 +661,7 @@ _PUBLIC_ struct tdb_context *tdb_open_ex(const char *name, int hash_size, int td
 	 * As this skips tdb->hdr_ofs.
 	 */
 	tdb->map_size = 0;
-	ret = tdb->methods->tdb_oob(tdb, 0, 1, 0);
+	ret = tdb_oob(tdb, 0, 1, 0);
 	if (ret == -1) {
 		errno = EIO;
 		goto fail;
@@ -687,6 +676,21 @@ _PUBLIC_ struct tdb_context *tdb_open_ex(const char *name, int hash_size, int td
 		}
 	}
 
+	if (tdb->hash_size > UINT32_MAX/4) {
+		TDB_LOG((tdb, TDB_DEBUG_FATAL, "tdb_open_ex: "
+			 "hash size %"PRIu32" too large\n", tdb->hash_size));
+		errno = EINVAL;
+		goto fail;
+	}
+
+	ret = tdb_oob(tdb, FREELIST_TOP, 4*tdb->hash_size, 1);
+	if (ret == -1) {
+		TDB_LOG((tdb, TDB_DEBUG_FATAL, "tdb_open_ex: "
+			 "hash size %"PRIu32" does not fit\n", tdb->hash_size));
+		errno = EINVAL;
+		goto fail;
+	}
+
 	if (locked) {
 		if (tdb_nest_unlock(tdb, ACTIVE_LOCK, F_WRLCK, false) == -1) {
 			TDB_LOG((tdb, TDB_DEBUG_ERROR, "tdb_open_ex: "
@@ -695,15 +699,19 @@ _PUBLIC_ struct tdb_context *tdb_open_ex(const char *name, int hash_size, int td
 			goto fail;
 		}
 
+
 	}
 
-	/* We always need to do this if the CLEAR_IF_FIRST flag is set, even if
-	   we didn't get the initial exclusive lock as we need to let all other
-	   users know we're using it. */
+	if (locked || (tdb_flags & TDB_CLEAR_IF_FIRST)) {
+		/*
+		 * We always need to do this if the CLEAR_IF_FIRST
+		 * flag is set, even if we didn't get the initial
+		 * exclusive lock as we need to let all other users
+		 * know we're using it.
+		 */
 
-	if (tdb_flags & TDB_CLEAR_IF_FIRST) {
-		/* leave this lock in place to indicate it's in use */
-		if (tdb_nest_lock(tdb, ACTIVE_LOCK, F_RDLCK, TDB_LOCK_WAIT) == -1) {
+		ret = tdb_nest_lock(tdb, ACTIVE_LOCK, F_RDLCK, TDB_LOCK_WAIT);
+		if (ret == -1) {
 			goto fail;
 		}
 	}
@@ -824,7 +832,7 @@ _PUBLIC_ int tdb_close(struct tdb_context *tdb)
 	return ret;
 }
 
-/* register a loging function */
+/* register a logging function */
 _PUBLIC_ void tdb_set_logging_function(struct tdb_context *tdb,
                                        const struct tdb_logging_context *log_ctx)
 {
@@ -893,7 +901,7 @@ static int tdb_reopen_internal(struct tdb_context *tdb, bool active_lock)
 	 * As this skips tdb->hdr_ofs.
 	 */
 	tdb->map_size = 0;
-	if (tdb->methods->tdb_oob(tdb, 0, 1, 0) != 0) {
+	if (tdb_oob(tdb, 0, 1, 0) != 0) {
 		goto fail;
 	}
 #endif /* fake pread or pwrite */
@@ -919,7 +927,10 @@ fail:
    seek pointer from our parent and to re-establish locks */
 _PUBLIC_ int tdb_reopen(struct tdb_context *tdb)
 {
-	return tdb_reopen_internal(tdb, tdb->flags & TDB_CLEAR_IF_FIRST);
+	bool active_lock;
+	active_lock = (tdb->flags & (TDB_CLEAR_IF_FIRST|TDB_MUTEX_LOCKING));
+
+	return tdb_reopen_internal(tdb, active_lock);
 }
 
 /* reopen all tdb's */
@@ -928,7 +939,10 @@ _PUBLIC_ int tdb_reopen_all(int parent_longlived)
 	struct tdb_context *tdb;
 
 	for (tdb=tdbs; tdb; tdb = tdb->next) {
-		bool active_lock = (tdb->flags & TDB_CLEAR_IF_FIRST);
+		bool active_lock;
+
+		active_lock =
+			(tdb->flags & (TDB_CLEAR_IF_FIRST|TDB_MUTEX_LOCKING));
 
 		/*
 		 * If the parent is longlived (ie. a
